#pragma once
#include <JUI/Base/Widget.hpp>


namespace JUI {

    /// An invisible, point-like widget which is used to attach decorator-type widgets at a given point.
    /// For example: The CurveSegment Widget uses two Anchors for it's endpoints.
    class Anchor : public Widget {
    public:
        Anchor() : Widget() {

        }
        explicit Anchor(Widget* parent) : Anchor() {
            Parent(parent);
        };
        explicit Anchor(Widget* parent, const UDim2& position) : Anchor(parent) {
            Position(position);
            Size({0_px, 0_px});
        }

        void Update(float elapsed) override {

        }

        void Draw() override {
            Color4 debug_color = Colors::Red;
            float debug_size = 1;

            J2D::FillCircle(debug_color, GetAbsolutePosition(), debug_size, 5);
        }

        Vector2 Point() { return GetAbsolutePosition(); }
    protected:
    private:
    };

    class Decorator : public Widget {
        public:
        Decorator() : Widget(){

        }
        explicit Decorator(Widget* parent) : Decorator() {
            Parent(parent);
        }

    };

    class ArrowDecorator : public Decorator {
    public:
        explicit ArrowDecorator(Widget* parent, Anchor* a, Anchor* b) : Decorator() {
            Parent(parent);
            origin = a;
            goal = b;
        }


        void Draw() override {

            auto goal_point = goal->Point();
            auto origin_point = origin->Point();

            using namespace J3ML;
            Color4 line_color = Colors::Blues::DarkBlue;
            float line_size = 1;

            J2D::DrawLine(line_color, origin->Point(), goal->Point(), line_size);

            float tri_size = 10;

            Vector2 triangle_base_pt = goal->Point() + (origin->Point().Normalized());

            J2D::DrawPoint(Colors::Yellow, triangle_base_pt, 4);

            Vector2 tri_left = triangle_base_pt.Normalized().Rotated90CW()*tri_size;
            Vector2 tri_right = triangle_base_pt.Normalized().Rotated90CW()*tri_size;
            Vector2 tri_top = goal->Point();

            J2D::DrawLine(line_color, tri_top, tri_left, line_size);
            J2D::DrawLine(line_color, tri_top, tri_right, line_size);

            J2D::FillTriangle(line_color, {tri_left, tri_right, tri_top});
        }
    protected:
        Anchor* origin;
        Anchor* goal;
    private:

    };
}
